searchData={"items":[{"type":"module","title":"SensitiveData","doc":"This library aims to make [data leak prevention](data_leak_prevention.html)\nstraightforward and convenient, by making it easy to follow most of the\n[Erlang Ecosystem Foundation](https://erlef.org/)'s recommendations regarding\n[protecting sensitive data](https://erlef.github.io/security-wg/secure_coding_and_deployment_hardening/sensitive_data.html).\n\nFor a quick overview, take a look at the [Getting Started](getting_started.html)\nand [Cheatsheet](cheatsheet.html) pages.\n\nWhile wrapping sensitive data within a `SensitiveData.Wrapper` instance will\nconform to many recommendations in the linked article (such as wrapping sensitive data in\na closure, pruning stack traces, and deriving the\n`Inspect` protocol), it doesn't cover others which may be relevant to your\nsituation (such as using the [:private option](https://erlang.org/doc/man/ets.html#new-2)\nfor ETS tables containing sensitive data, flagging the current process as\nsensitive using [:erlang.process_flag(:sensitive, true)](https://erlang.org/doc/man/erlang.html#process_flag-2)\nin processes holding sensitive data, and so on).\n\n> ##","ref":"SensitiveData.html"},{"type":"module","title":"Beware {: .warning} - SensitiveData","doc":">\n> This library is intended to prevent accidental leakage of sensitive data, it **WILL NOT**\n> keep data safe from a malicious actor that has access to your system. For example, the\n> wrapped sensitive data could be read via BEAM introspection functionality, such as\n> [`:erlang.fun_info(..., :env)`](https://www.erlang.org/doc/man/erlang#fun_info-2)","ref":"SensitiveData.html#module-beware-warning"},{"type":"function","title":"SensitiveData.exec/2","doc":"Executes the provided function, ensuring no data leaks in case of error.","ref":"SensitiveData.html#exec/2"},{"type":"function","title":"Options - SensitiveData.exec/2","doc":"- `:into` - a `t:SensitiveData.Wrapper.spec/0` value into which the `fun` execution\n  result should be wrapped.\n- `:exception_redactor` - the `t:SensitiveData.Redaction.exception_redactor/0`\n  to use when redacting an `t:Exception.t/0`. Defaults to\n  `SensitiveData.Redactors.Exception.drop/1`, which is also the fallback if\n  the custom redactor fails. See\n  [Custom Failure Redaction](SensitiveData.Wrapper.html#module-custom-failure-redaction).\n- `:stacktrace_redactor` - the `t:SensitiveData.Redaction.stacktrace_redactor/0`\n  to use when redacting a stack trace. Defaults to\n  `SensitiveData.Redactors.Stacktrace.strip/1`, which is also the fallback if\n  the custom redactor fails. See\n  [Custom Failure Redaction](SensitiveData.Wrapper.html#module-custom-failure-redaction).","ref":"SensitiveData.html#exec/2-options"},{"type":"function","title":"Examples - SensitiveData.exec/2","doc":"iex> Map.get(\"SOME_PASSWORD\", :foobar)\n    ** (BadMapError) expected a map, got: \"SOME_PASSWORD\"\n\n    iex> SensitiveData.exec(fn ->\n    ...>   Map.get(\"SOME_PASSWORD\", :foobar)\n    ...> end)\n    ** (SensitiveData.RedactedException) an exception of type `BadMapError` was raised in a sensitive context\n\nPassing the execution result to a `SecretData` module implementing\nthe `SensitiveData.Wrapper` behaviour:\n\n    SensitiveData.exec(fn ->\n      System.fetch_env!(\"DATABASE_PASSWORD\")\n    end, into: SecretData)\n    #SecretData<...>","ref":"SensitiveData.html#exec/2-examples"},{"type":"function","title":"SensitiveData.gets_sensitive/2","doc":"Reads a line from stdin, without echoing the input back to the console.","ref":"SensitiveData.html#gets_sensitive/2"},{"type":"function","title":"Options - SensitiveData.gets_sensitive/2","doc":"- `:into` - a `t:SensitiveData.Wrapper.spec/0` value into which the input should\n  be wrapped. By default, the input is not wrapped and is returned as-is, similarly\n  to `IO.gets/2`.","ref":"SensitiveData.html#gets_sensitive/2-options"},{"type":"function","title":"Examples - SensitiveData.gets_sensitive/2","doc":"To display a prompt and await user input:\n\n    SensitiveData.gets_sensitive(\"Enter your database password: \")\n\nTo do the same but wrap the result within a `SecretData` module implementing\nthe `SensitiveData.Wrapper` behaviour:\n\n    SensitiveData.gets_sensitive(\"Enter your database password: \",\n      into: SecretData)","ref":"SensitiveData.html#gets_sensitive/2-examples"},{"type":"type","title":"SensitiveData.exec_opts/0","doc":"","ref":"SensitiveData.html#t:exec_opts/0"},{"type":"module","title":"SensitiveData.Guards","doc":"Guards for sensitive data wrappers.\n\nThis module includes guards that work with `t:SensitiveData.Wrapper.t/0` wrappers. For example:\n\n    import SensitiveData.Guards, only: [is_sensitive_binary: 1]\n\n    def send_request(api_key) when is_sensitive_binary(api_key) do\n      ...\n    end\n\nFor more information on guards, refer to Elixir's\n[Patterns and guards](https://hexdocs.pm/elixir/patterns-and-guards.html) page.","ref":"SensitiveData.Guards.html"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0`;\notherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive/2","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` and `struct_name`\nis the wrapper implementation module; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive/2"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_atom/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing an\natom; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_atom/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_binary/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing a\nbinary; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_binary/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_bitstring/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing a\nbitstring; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_bitstring/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_boolean/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing a\nboolean; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_boolean/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_exception/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing an\nexception; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_exception/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_exception/2","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing an\nexception of `name`; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_exception/2"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_float/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing a\nfloat; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_float/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_function/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing a\nfunction; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_function/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_function/2","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing a\nfunction that can be applied with `arity` number of arguments; otherwise\nreturns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_function/2"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_integer/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing an\ninteger; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_integer/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_list/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing a list;\notherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_list/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_map/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing a map;\notherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_map/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_nil/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing `nil`;\notherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_nil/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_number/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing an\ninteger or a float; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_number/1"},{"type":"macro","title":"SensitiveData.Guards.is_sensitive_tuple/1","doc":"Returns `true` if `term` is a `t:SensitiveData.Wrapper.t/0` containing a\ntuple; otherwise returns false.\n\nAllowed in guard tests.","ref":"SensitiveData.Guards.html#is_sensitive_tuple/1"},{"type":"behaviour","title":"SensitiveData.Wrapper","doc":"Defines a wrapper for sensitive data.\n\n[//]: # (This is used in an HTML anchor: if updated, update links with)\n[//]: # (#module-using)","ref":"SensitiveData.Wrapper.html"},{"type":"behaviour","title":"Using - SensitiveData.Wrapper","doc":"When used, this module will implement the callbacks from this\n`SensitiveData.Wrapper` module, making the module where the `use`\ncall is made into a sensitive data wrapper.\n\n```\ndefmodule MySensitiveData do\n  use SensitiveData.Wrapper\nend\n```\n\nThe options when using are:\n\n- `:allow_label` - a boolean indicating whether the `:label` option\n  is allowed on instance wrappers (see for example `c:from/2`).\n  Defaults to `false`.\n  This option should be used with care, see\n  [Redacting and Labeling](#module-redacting-and-labeling))\n- `:redactor` - a `t:function_handle/0` pointing to a\n  `t:SensitiveData.Redaction.redactor/1` function (for any wrapped\n  `t:term/0`) able to appropriately redact the wrapped sensitive\n  term for display (such as via `inspect/1`).\n  By default, no redacted value is made available.\n  If the redactor function fails (e.g., by raising), the redacted\n  value will be set to `SensitiveData.Redacted`.\n  This option should be used with care, see\n  [redacting and labeling section](#module-redacting-and-labeling))\n- `:exception_redactor` - a `t:function_handle/0` pointing to a\n  `t:SensitiveData.Redaction.exception_redactor/0`.\n  By default, exception redaction is provided by `SensitiveData.Redactors.Exception.drop/1`.\n  This option should be used with care, see [Custom Failure Redaction](#module-custom-failure-redaction)\n- `:stacktrace_redactor` - a `t:function_handle/0` pointing to a\n  `t:SensitiveData.Redaction.stacktrace_redactor/0`.\n  By default, stack trace redaction is provided by `SensitiveData.Redactors.Stacktrace.strip/1`.\n  This option should be used with care, see [Custom Failure Redaction](#module-custom-failure-redaction)\n- `:wrap` - a boolean indicating whether the `c:wrap/2` callback implementation\n  should be generated. Defaults to `false`.\n- `:unwrap` - a boolean indicating whether the `c:unwrap/1` callback implementation\n  should be generated. Defaults to `false`.\n\n[//]: # (This is used in an HTML anchor: if updated, update links with)\n[//]: # (#module-redacting-and-labeling)","ref":"SensitiveData.Wrapper.html#module-using"},{"type":"behaviour","title":"Redacting and Labeling - SensitiveData.Wrapper","doc":"It can be helpful to have some contextual information about the sensitive data contained\nwithin a wrapper. Aside from [guards](SensitiveData.Guards.html), you may wish to make\nuse of:\n- redaction at the module level (i.e., single shared redaction logic for all terms\n  wrapped by the same module)\n- labels at the instance level (i.e., each wrapper instance can have its own different\n  label)\n\n> ##","ref":"SensitiveData.Wrapper.html#module-redacting-and-labeling"},{"type":"behaviour","title":"Beware {: .warning} - SensitiveData.Wrapper","doc":">\n> Redacting and labeling should be used with utmost care to ensure they won't leak any\n> sensitive data under any circumstances.\n>\n> **If you use a custom redaction strategy**, you must ensure it won't leak sensitive\n> information for any possible sensitive term wrapped by the module.\n>\n> **If you allow labeling**, you must ensure that any call site setting a label is doing so\n> without leaking sensitive data.\n\n#","ref":"SensitiveData.Wrapper.html#module-beware-warning"},{"type":"behaviour","title":"Examples - SensitiveData.Wrapper","doc":"defmodule CreditCard do\n      use SensitiveData.Wrapper, allow_label: true, redactor: :redactor\n\n      def redactor(card_number) when is_binary(card_number) do\n        {< >, last_four} = String.split_at(card_number, -4)\n        IO.iodata_to_binary([first_number, List.duplicate(\"*\", String.length(to_mask)), last_four])\n      end\n    end\n\n    # in IEx:\n    CreditCard.from(fn -> \"5105105105105100\" end, label: {:type, :debit})\n    #CreditCard \n\nBoth the redacted value and the label will be maintained as fields within\nthe wrapper\nand can be used to assist in determining what the wrapped sensitive value\nwas then the wrapper is inspected (manually when debugging, via Observer,\ndumped in crashes, and so on). Both values can be used in pattern matches.\n\nFor both redacting and labeling, `nil` values will not be displayed when\ninspecting.\n\n[//]: # (This is used in an HTML anchor: if updated, update links with)\n[//]: # (#module-custom-failure-redaction)","ref":"SensitiveData.Wrapper.html#module-examples"},{"type":"behaviour","title":"Custom Failure Redaction - SensitiveData.Wrapper","doc":"If an exception is raised within a sensitive context (such within `c:exec/3`, `c:from/2`,\nor `c:map/3`), both the exception and stack trace will be redacted. By default:\n\n- the exception will be redacted with `SensitiveData.Redactors.Exception.drop/1`\n- the stack trace will be redacted with `SensitiveData.Redactors.Stacktrace.strip/1`\n\nHowever, failure redaction can be customized via the `:exception_redactor` and\n`:stacktrace_redactor` options given to the `use` call.\n\n> ##","ref":"SensitiveData.Wrapper.html#module-custom-failure-redaction"},{"type":"behaviour","title":"Beware {: .warning} - SensitiveData.Wrapper","doc":">\n> Custom failure redaction should be used with utmost care to ensure it won't leak any\n> sensitive data under any circumstances.\n>\n> For exception redaction, you must ensure it won't leak sensitive\n> information for any possible exception: standard Elixir ones, the ones in your code base,\n> but also any exception that may be raised from a dependency.\n>\n> For stack trace redaction, it must handle all possible stack traces.\n\nIf a custom redactor function fails, redaction will fall back to the corresponding default\nredactor listed above.","ref":"SensitiveData.Wrapper.html#module-beware-warning"},{"type":"callback","title":"SensitiveData.Wrapper.exec/3","doc":"Returns the result of the callback invoked with the sensitive term.\n\nExecutes the provided function with the sensitive term provided as the function argument,\nensuring no data leaks in case of error.\n\nThe unwrapped result of the callback execution is then either returned as is,\nor wrapped according to provided options.","ref":"SensitiveData.Wrapper.html#c:exec/3"},{"type":"callback","title":"Options - SensitiveData.Wrapper.exec/3","doc":"See `t:exec_opts/0`.","ref":"SensitiveData.Wrapper.html#c:exec/3-options"},{"type":"callback","title":"Examples - SensitiveData.Wrapper.exec/3","doc":"# CreditCard implements the SensitiveData.Wrapper behaviour\n    credit_card = CreditCard.from(fn -> \"5105105105105100\" end)\n\n    # We can call a function that expect a credit card number (in string\n    # format), and will return `%{result: :ok}` upon successful payment:\n    %{result: :ok} = CreditCard.exec(credit_card, &pay_with_credit_card/1)\n\n    # We can also alter the wrapped data without ever exposing it outside\n    # of a sensitive context protecting the data from leaks:\n    # PaymentToken implements the SensitiveData.Wrapper behaviour\n    CreditCard.exec(credit_card, fn card_number -> tokenize(card_number) end, into: PaymentToken)\n    # #PaymentToken<...>","ref":"SensitiveData.Wrapper.html#c:exec/3-examples"},{"type":"callback","title":"SensitiveData.Wrapper.from/2","doc":"Wraps the sensitive term returned by the callback to prevent unwanted data\nleaks.\n\nThe callback is executed only once: during wrapper instanciation.","ref":"SensitiveData.Wrapper.html#c:from/2"},{"type":"callback","title":"Options - SensitiveData.Wrapper.from/2","doc":"See `t:wrap_opts/0`.","ref":"SensitiveData.Wrapper.html#c:from/2-options"},{"type":"callback","title":"Examples - SensitiveData.Wrapper.from/2","doc":"MySensitiveData.from(fn -> \"foo\" end)\n    # #MySensitiveData<...>\n\n    MySensitiveData.from(fn -> \"5105105105105100\" end, label: :credit_card_user_bob)\n    # #MySensitiveData","ref":"SensitiveData.Wrapper.html#c:from/2-examples"},{"type":"callback","title":"SensitiveData.Wrapper.map/3","doc":"Invokes the callback on the wrapped sensitive term and returns the wrapped result.","ref":"SensitiveData.Wrapper.html#c:map/3"},{"type":"callback","title":"Options - SensitiveData.Wrapper.map/3","doc":"See `t:wrap_opts/0`.","ref":"SensitiveData.Wrapper.html#c:map/3-options"},{"type":"callback","title":"Examples - SensitiveData.Wrapper.map/3","doc":"data = MySensitiveData.from(fn -> \"foo\" end)\n\n    MySensitiveData.map(data, fn orig -> orig <> \"bar\" end, label: :now_foobar)\n    # #MySensitiveData","ref":"SensitiveData.Wrapper.html#c:map/3-examples"},{"type":"function","title":"SensitiveData.Wrapper.sensitive_length/1","doc":"Returns the length of the list wrapped within `term`.","ref":"SensitiveData.Wrapper.html#sensitive_length/1"},{"type":"function","title":"SensitiveData.Wrapper.sensitive_map_size/1","doc":"Returns the size of the map wrapped within `term`.\n\nThe size of a map is the number of key-value pairs that the map contains.\n\nThis operation happens in constant time.","ref":"SensitiveData.Wrapper.html#sensitive_map_size/1"},{"type":"function","title":"SensitiveData.Wrapper.sensitive_tuple_size/1","doc":"Returns the size of a tuple wrapped within `term`.\n\nThis operation happens in constant time.","ref":"SensitiveData.Wrapper.html#sensitive_tuple_size/1"},{"type":"callback","title":"SensitiveData.Wrapper.unwrap/1","doc":"Returns the sensitive term within `wrapper`.\n\nBy default, this optional callback will not be generated when `SensitiveData.Wrapper`\nis `use`d.\n\n> #","ref":"SensitiveData.Wrapper.html#c:unwrap/1"},{"type":"callback","title":"Avoid Unwrapping Sensitive Data {: .warning} - SensitiveData.Wrapper.unwrap/1","doc":">\n> Calling this function should be discouraged: `c:exec/3` should be used instead\n> to interact with sensitive data.\n\nYou can always obtain the raw sensitive data via `exec(wrapped_value, & &1)` but should seriously\nreconsider if that's needed: usually a combination of `map/2` and `exec/2` should\nsatisfy all your needs regarding sensitive data interaction, and sensitive data\ntypically never needs to be extracted from wrappers.","ref":"SensitiveData.Wrapper.html#c:unwrap/1-avoid-unwrapping-sensitive-data-warning"},{"type":"callback","title":"Examples - SensitiveData.Wrapper.unwrap/1","doc":"data = MySensitiveData.from(fn -> \"foo\" end)\n\n    # Not recommended\n    MySensitiveData.unwrap(data)\n    # \"foo\"\n\n    # Do this instead if you absolutely must unwrap the value\n    MySensitiveData.exec(data, & &1)\n    # \"foo\"","ref":"SensitiveData.Wrapper.html#c:unwrap/1-examples"},{"type":"callback","title":"SensitiveData.Wrapper.wrap/2","doc":"Wraps the sensitive `term` to prevent unwanted data leaks.\n\n> #","ref":"SensitiveData.Wrapper.html#c:wrap/2"},{"type":"callback","title":"Prefer Using From {: .tip} - SensitiveData.Wrapper.wrap/2","doc":">\n> Calling this function should be discouraged: `c:from/2` should be used instead\n> to wrap sensitive data, as in all cases where you would call `wrap(my_value)`\n> you should instead call `from(fn -> my_value end)`.\n\nThe reason for this preference for `from/2` is that it is much harder to\naccidentally misuse. For example, `wrap(get_credit_card_number())` and\n`from(get_credit_card_number)` look very similar, but if `get_credit_card_number/0`\nraises and leaks sensitive data when it does, the call to `wrap/1` will raise and\nexpose sensitive information whereas the call to `from/1` will not. This is because\n`from/2` internally wraps callback execution within `SensitiveData.exec/2`.\n\nAdditionally, making use of `wrap/2` means you have access to unwrapped sensitive\ndata, which should be discouraged: regardless of whether this sensitive data was\nfetched, generated, or derived, this should be done via functions from this library.\nPut another way, obtaining sensitive data and only then stuffing it into a wrapper\nis an anti-pattern that shouldn't be encouraged, and `wrap/2` facilitates it.","ref":"SensitiveData.Wrapper.html#c:wrap/2-prefer-using-from-tip"},{"type":"callback","title":"Options - SensitiveData.Wrapper.wrap/2","doc":"See `t:wrap_opts/0`","ref":"SensitiveData.Wrapper.html#c:wrap/2-options"},{"type":"callback","title":"Examples - SensitiveData.Wrapper.wrap/2","doc":"# Not recommended\n    MySensitiveData.wrap(\"foo\")\n    # #MySensitiveData<...>\n\n    # Do this instead\n    MySensitiveData.from(fn -> \"foo\" end)\n    # #MySensitiveData<...>\n\n    # Not recommended\n    MySensitiveData.wrap(\"5105105105105100\", label: :credit_card_user_bob)\n    # #MySensitiveData \n\n    # Do this instead\n    MySensitiveData.from(fn -> \"5105105105105100\" end, label: :credit_card_user_bob)\n    # #MySensitiveData","ref":"SensitiveData.Wrapper.html#c:wrap/2-examples"},{"type":"type","title":"SensitiveData.Wrapper.exec_opts/0","doc":"Execution options.","ref":"SensitiveData.Wrapper.html#t:exec_opts/0"},{"type":"type","title":"Options - SensitiveData.Wrapper.exec_opts/0","doc":"- `:into` - a `t:spec/0` value defining how the execution result should be wrapped.","ref":"SensitiveData.Wrapper.html#t:exec_opts/0-options"},{"type":"type","title":"SensitiveData.Wrapper.function_handle/0","doc":"The location of a function.\n\nIf provided as an `atom`, the function is that with the same name and located in the\ncurrent module.\n\nIf provided as a `{module, atom}` tuple, the function is that with the same name as `atom`\nand located in the `module` module.","ref":"SensitiveData.Wrapper.html#t:function_handle/0"},{"type":"type","title":"SensitiveData.Wrapper.spec/0","doc":"A wrapper target.\n\nIf provided as a `wrapper_module` module name, the `c:from/2` callback in the\ncorresponding wrapper module will be called with default options.\n\nIf provided as a `{wrapper_module, wrap_opts}` tuple, the `c:from/2` callback in the\ncorresponding wrapper module will be called with the provided `wrap_opts` options.","ref":"SensitiveData.Wrapper.html#t:spec/0"},{"type":"type","title":"SensitiveData.Wrapper.t/0","doc":"A wrapper containing sensitive data.\n\nThe wrapper structure should be considered opaque, aside from the `label` and\n`redacted` fields (see\n[redacting and labeling section](#module-redacting-and-labeling)). You may\nread and match on those fields, but accessing any other fields or directly\nmodifying any field is not advised.\n\nLimited information regarding the contained sensitive data can be obtained\nvia the guards in `SensitiveData.Guards` and the functions in\n`SensitiveData.Wrapper.Util`.","ref":"SensitiveData.Wrapper.html#t:t/0"},{"type":"type","title":"SensitiveData.Wrapper.wrap_opts/0","doc":"Wrapping options.\n\nAllowable options are configured during `use` invocation, see\n[Using section](#module-using).\n\nInvalid or unsupported values will be ignored and logged.","ref":"SensitiveData.Wrapper.html#t:wrap_opts/0"},{"type":"type","title":"Options - SensitiveData.Wrapper.wrap_opts/0","doc":"- `:label` - a label displayed when the wrapper is inspected. This option is only\n  available if the `:allow_label` option was set to `true` when [using](#module-using)\n  `SensitiveData.Wrapper`.","ref":"SensitiveData.Wrapper.html#t:wrap_opts/0-options"},{"type":"type","title":"SensitiveData.Wrapper.wrapper_module/0","doc":"A module implementing the `SensitiveData.Wrapper` behaviour.","ref":"SensitiveData.Wrapper.html#t:wrapper_module/0"},{"type":"module","title":"SensitiveData.Redacted","doc":"An empty module serving only to indicate that data has been redacted.\n\nThis typically shows up in redacted exceptions and stack traces,\nas well as in cases where a `SensitiveData.Wrapper`'s custom `:redactor`\nfunction fails.","ref":"SensitiveData.Redacted.html"},{"type":"module","title":"SensitiveData.Redaction","doc":"Utilities for redacting information from exceptions and stack traces.","ref":"SensitiveData.Redaction.html"},{"type":"type","title":"SensitiveData.Redaction.exception_redactor/0","doc":"A function responsible for redacting exceptions.\n\nThe function must return a redacted version of the provided exception.\n\nIf this function fails, redaction will fall back to\n`SensitiveData.Redactors.Exception.drop/1`.\n\n> ##","ref":"SensitiveData.Redaction.html#t:exception_redactor/0"},{"type":"type","title":"Beware {: .warning} - SensitiveData.Redaction.exception_redactor/0","doc":">\n> If you use a custom exception redaction strategy, you must ensure it won't leak sensitive\n> information for any possible exception provided as the argument.\n>\n> Bear in mind you must handle not only standard Elixir exceptions and those from your code\n> base, but also any exception that may be raised by any dependency.","ref":"SensitiveData.Redaction.html#t:exception_redactor/0-beware-warning"},{"type":"type","title":"SensitiveData.Redaction.redactor/1","doc":"","ref":"SensitiveData.Redaction.html#t:redactor/1"},{"type":"type","title":"SensitiveData.Redaction.stacktrace_redactor/0","doc":"A function responsible for redacting a stacktrace.\n\nThe function must return a redacted version of the provided stack trace.\n\nIf this function fails, redaction will fall back to\n`SensitiveData.Redactors.Stacktrace.strip/1`.\n\n> ##","ref":"SensitiveData.Redaction.html#t:stacktrace_redactor/0"},{"type":"type","title":"Beware {: .warning} - SensitiveData.Redaction.stacktrace_redactor/0","doc":">\n> If you use a custom stack trace redaction strategy, you must ensure it won't leak sensitive\n> data under any circumstances.","ref":"SensitiveData.Redaction.html#t:stacktrace_redactor/0-beware-warning"},{"type":"module","title":"SensitiveData.Redactors.Exception","doc":"Functions for redacting exceptions.\n\n> #","ref":"SensitiveData.Redactors.Exception.html"},{"type":"module","title":"Tip {: .tip} - SensitiveData.Redactors.Exception","doc":">\n> You typically won't need to concern yourself with functions in this module,\n> as `SensitiveData.exec/2`, `c:SensitiveData.Wrapper.from/2`, and\n> friends, will use safe redaction implementations by default.","ref":"SensitiveData.Redactors.Exception.html#module-tip-tip"},{"type":"function","title":"SensitiveData.Redactors.Exception.drop/1","doc":"Replaces the exception with a fully redacted exception.","ref":"SensitiveData.Redactors.Exception.html#drop/1"},{"type":"function","title":"Example - SensitiveData.Redactors.Exception.drop/1","doc":"iex(1)> e = ArgumentError.exception(\"foo\")\n    %ArgumentError{message: \"foo\"}\n    iex(2)> SensitiveData.Redactors.Exception.drop(e)\n    %SensitiveData.RedactedException{exception_name: ArgumentError}","ref":"SensitiveData.Redactors.Exception.html#drop/1-example"},{"type":"module","title":"SensitiveData.Redactors.Stacktrace","doc":"Functions for redacting stack traces.\n\n> #","ref":"SensitiveData.Redactors.Stacktrace.html"},{"type":"module","title":"Tip {: .tip} - SensitiveData.Redactors.Stacktrace","doc":">\n> You typically won't need to concern yourself with functions in this module,\n> as `SensitiveData.exec/2`, `c:SensitiveData.Wrapper.from/2`, and\n> friends, will use safe redaction implementations by default.","ref":"SensitiveData.Redactors.Stacktrace.html#module-tip-tip"},{"type":"function","title":"SensitiveData.Redactors.Stacktrace.strip/1","doc":"Redacts the argument list in the stack trace.","ref":"SensitiveData.Redactors.Stacktrace.html#strip/1"},{"type":"function","title":"Example - SensitiveData.Redactors.Stacktrace.strip/1","doc":"iex(1)> stacktrace =\n    ...(1)>   try do\n    ...(1)>     Map.get(\"SOME SECRET\", :some_key)\n    ...(1)>   rescue\n    ...(1)>     _ -> __STACKTRACE__\n    ...(1)>   end\n    iex(2)> true = match?([{Map, :get, [\"SOME SECRET\", :some_key, nil], _} | _], stacktrace)\n    iex(3)> [_first_list | rest] = stacktrace\n    iex(4)> redacted_stacktrace = SensitiveData.Redactors.Stacktrace.strip(stacktrace)\n    iex(5)> true = match?([{Map, :get, 3, _} | ^rest], redacted_stacktrace)","ref":"SensitiveData.Redactors.Stacktrace.html#strip/1-example"},{"type":"exception","title":"SensitiveData.RedactedException","doc":"A fully redacted exception.\n\nThis is used to replace exceptions raised in a sensitive context,\nsuch as within `SensitiveData.exec/2`.","ref":"SensitiveData.RedactedException.html"},{"type":"function","title":"SensitiveData.RedactedException.message/1","doc":"","ref":"SensitiveData.RedactedException.html#message/1"},{"type":"type","title":"SensitiveData.RedactedException.t/0","doc":"","ref":"SensitiveData.RedactedException.html#t:t/0"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nThis short guide will help you get up to speed with using this library. It will\nuse API credentials and interactation as a guiding example, but the details\ndon't matter.\n\nFor context, the relevant things to know about this example are:\n\n- API username and password are available in [environment variables](https://en.wikipedia.org/wiki/Environment_variable)\n    and are considered to be highly sensitive\n- API authentication happens via [basich auth](https://en.wikipedia.org/wiki/Basic_access_authentication)\n- `fetch_data_from_api` makes a call to the authenticated API, for which it\n    requires access credentials already in the basic auth format","ref":"getting_started.html"},{"type":"extras","title":"Create a Wrapper Module - Getting Started","doc":"First we need to define a wrapper module to hold sensitive data:\n\n```elixir\ndefmodule MyApp.SecretData do\n  use SensitiveData.Wrapper\nend\n```\n\nThis `MyApp.SecretData` module implements the `SensitiveData.Wrapper`\nbehaviour, so feel free to read more about it in the documentation.","ref":"getting_started.html#create-a-wrapper-module"},{"type":"extras","title":"Wrap Sensitive Data - Getting Started","doc":"Let's fetch the sensitive credentials from the environment, and convert them\ninto a more convenient form:\n\n```elixir\nalias MyApp.SecretData\n\napi_credentials =\n  SecretData.from(fn ->\n    user = System.fetch_env!(\"API_USER\")\n    password = System.fetch_env!(\"API_PASSWORD\")\n    Base.encode64(\"#{user}:#{password}\")\n  end)\n```","ref":"getting_started.html#wrap-sensitive-data"},{"type":"extras","title":"Interact with Wrapped Data - Getting Started","doc":"With our credentials available in a wrapper, we can now make use of this\nsensitive information safely via `c:SensitiveData.Wrapper.exec/3`:\n\n```elixir\nalias MyApp.SecretData\n\n{:ok, _api_call_result} =\n  SecretData.exec(api_credentials, fn basic_auth ->\n    fetch_data_from_api(basic_auth: basic_auth)\n  end)\n```\n\nThis way if there's any issue, we can be sure that no sensitive information\n(e.g., API authentication credentials) will leak through stack traces,\ncrash dumps, runtime state inspection, and so on.\n\n<!-- It will\nuse a database credentials and interactation as a guiding example, but the\ndetails don't matter: you should be able to follow along even if you've never\nused a database (much less `Postgrex`) or even know what a database does. -->\n\n<!-- For context, the relevant things to know about this example are:\n\n- the information in the \"DB_URI\" [environment variable](https://en.wikipedia.org/wiki/Environment_variable)\n    is highly sensitive as it could give access to confidential information\n    stored in the database if the value were read\n- the connection information is formatted as a [URI](https://en.wikipedia.org/wiki/Uniform_Resource_Identifier),\n    but the function to connect to the database requires the configuration to\n    be provided as a keyword list: we'll need to parse the URI before we'll\n    be able to connect to the database. The actual parsing will be done by\n    `parse_postgres_uri` in our example. Just like any function,\n    `parse_postgres_uri` can fail and may expose sensitive information (such as\n    the database password) in a stack trace.","ref":"getting_started.html#interact-with-wrapped-data"},{"type":"extras","title":"Create a Wrapper Module - Getting Started","doc":"First we need to define a wrapper module to hold sensitive data:\n\n```elixir\ndefmodule MyApp.SecretData do\n  use SensitiveData.Wrapper\nend\n```\n\nThis `MyApp.SecretData` module implements the `SensitiveData.Wrapper`\nbehaviour, so feel free to read more about it in the documentation.","ref":"getting_started.html#create-a-wrapper-module"},{"type":"extras","title":"Wrap Sensitive Data - Getting Started","doc":"```elixir\nalias MyApp.SecretData\n\ndatabase_uri = SecretData.from(fn -> System.get_env(\"DB_URI\") end)\n```","ref":"getting_started.html#wrap-sensitive-data"},{"type":"extras","title":"Interact with Wrapped Data - Getting Started","doc":"```elixir\nalias MyApp.SecretData\n\n{:ok, pid} =\n  SecretData.exec(database_uri, fn uri ->\n    uri\n    |> parse_postgres_uri()\n    |> Postgrex.start_link()\n  end)\n``` -->","ref":"getting_started.html#interact-with-wrapped-data"},{"type":"extras","title":"Cheatsheet","doc":"# Cheatsheet","ref":"cheatsheet.html"},{"type":"extras","title":"Creating a Wrapper - Cheatsheet","doc":"```\ndefmodule MySecretData do\n  use SensitiveData.Wrapper\nend\n```","ref":"cheatsheet.html#creating-a-wrapper"},{"type":"extras","title":"Getting Data into a Wrapper - Cheatsheet","doc":"{:.col-2}\n\n#","ref":"cheatsheet.html#getting-data-into-a-wrapper"},{"type":"extras","title":"SensitiveData `exec/2` - Cheatsheet","doc":"```\nSensitiveData.exec(\n  fn -> System.fetch_env!(\"DB_URI\") end,\n  into: MySecretData)\n```\n\n#","ref":"cheatsheet.html#sensitivedata-exec-2"},{"type":"extras","title":"Wrapper `from/2` - Cheatsheet","doc":"```\nMySecretData.from(fn -> System.fetch_env!(\"DB_URI\") end)\n```\n\n#","ref":"cheatsheet.html#wrapper-from-2"},{"type":"extras","title":"From `:stdio` - Cheatsheet","doc":"```\nSensitiveData.gets_sensitive(\"Your password: \",\n  into: MySecretData)\n```\n\nor alternatively\n\n```\nMySecretData.from(fn ->\n  SensitiveData.gets_sensitive(\"Your password: \")\nend)\n```","ref":"cheatsheet.html#from-stdio"},{"type":"extras","title":"Interacting with Wrapped Data - Cheatsheet","doc":"{:.col-2}\n\n### `map/3`\n\nThe sensitive data within a container can easily be modified using `c:SensitiveData.Wrapper.map/3`:\n\n```\n# get authentication data in Basic Authentication format\nbasic_credentials = MySecretData.from(fn ->\n  System.fetch_env!(\"BASIC_AUTH\") end)\n\n# later, these credentials need to be converted\n# to username & password\nmap_credentials =\n  MySecretData.map(basic_credentials, fn basic ->\n    [username, password] =\n      basic\n      |> Base.decode64!()\n      |> String.split(\":\")\n  end)\n```\n\n### `exec/3`\n\nExecuting functions requiring sensitive data located in a wrapper can be\naccomplished with `c:SensitiveData.Wrapper.exec/3`:\n\n```\n# get authentication data\ncredentials = MySecretData.from(fn ->\n  System.fetch_env!(\"BASIC_AUTH\") end)\n\n# later, make an API request using these credentials\n{:ok, data} =\n  MySecretData.exec(credentials, fn basic_auth ->\n    get_api_results(\"/some/endpoint\", basic_auth)\n  end)\n```","ref":"cheatsheet.html#interacting-with-wrapped-data"},{"type":"extras","title":"Identifying Wrapper Contents - Cheatsheet","doc":"It is often useful to be able to determine some information about the sensitive\ndata contained within wrappers, both for programmatic and human debuggability\nreasons.\n\n#","ref":"cheatsheet.html#identifying-wrapper-contents"},{"type":"extras","title":"Guards and Utilities - Cheatsheet","doc":"Functions from the `SensitiveData.Guards` module and utility functions from\nthe `SensitiveData.Wrapper` module can help with branching logic depending\n(somewhat) on the sensitive data within the wrappers:\n\n```\nrequire SensitiveData.Guards\n\nalias SensitiveData.Guards\nalias SensitiveData.Wrapper\n\ndata = MySecretData.from(fn ->\n  %{foo: :yes, bar: :no} end)\n\ncase data do\n  map when Guards.is_sensitive_map(map) ->\n    IO.puts(\"It's a map with #{Wrapper.sensitive_map_size(map)} items\")\n\n  _ ->\n    IO.puts(\"It's not a map\")\nend\n```\n\n#","ref":"cheatsheet.html#guards-and-utilities"},{"type":"extras","title":"Redaction - Cheatsheet","doc":"It can be useful to have a redacted version of sensitive data, which can be defined at the module\nlevel with the `redactor` option given to `use SensitiveData.Wrapper`, which can be given as:\n\n- an atom - the name of the redactor function located in the same module\n- a `{Module, func}` tuple - the redactor function `func` from module `Module` will be used for redaction\n\nIn either case, the redaction function will be called with a single argument: the sensitive term.\n\n⚠️️ **BEWARE** ⚠️ If you use a custom redaction strategy, you must ensure it won't leak any\nsensitive data under any circumstances.\n\n\n```\ndefmodule CreditCard do\n  use SensitiveData.Wrapper, redactor: :redactor\n\n  def redactor(card_number) when is_binary(card_number) do\n    {< >, last_four} = String.split_at(card_number, -4)\n\n    IO.iodata_to_binary([first_number, List.duplicate(\"*\", String.length(to_mask)), last_four])\n  end\nend\n```\n\nAlternatively, the same result can be achieved with:\n\n```\ndefmodule MyApp.Redaction do\n  def redact_credit_card(card_number) when is_binary(card_number) do\n    {< >, last_four} = String.split_at(card_number, -4)\n\n    IO.iodata_to_binary([first_number, List.duplicate(\"*\", String.length(to_mask)), last_four])\n  end\nend\n\ndefmodule CreditCard do\n  use SensitiveData.Wrapper, redactor: {MyApp.Redaction, :redact_credit_card}\nend\n```\n\nIn use (either implementation):\n\n```shell\niex(1)> cc = CreditCard.from(fn -> \"5105105105105100\" end)\n#CreditCard \niex(2)> cc.redacted\n\"5***********5100\"\n```\n\n#","ref":"cheatsheet.html#redaction"},{"type":"extras","title":"Labeling - Cheatsheet","doc":"It can be useful to label sensitive data, for example to give scope to the given data. Labeling\nmust be explicitly enabled via the `allow_label: true` option given to `use SensitiveData.Wrapper`.\n\nOnce enabled at the module level, each instance of sensitive data can be labeled via the `:label` option.\n\n⚠️ **BEWARE** ⚠️ If you allow labels, you must ensure call sites aren't leaking\nsensitive data via label values.\n\n\n```\ndefmodule DatabaseCredentials do\n  use SensitiveData.Wrapper, allow_label: true\nend\n```\n\nIn use:\n\n```\niex(1)> db = DatabaseCredentials.from(fn -> System.fetch_env(\"PROD_DB_CREDS\") end, label: :prod)\n#DatabaseCredentials \niex(2)> db.label\n:prod\niex(3)> DatabaseCredentials.from(fn -> System.fetch_env(\"PROD_CI_CREDS\") end, label: :integration)\n#DatabaseCredentials \n```\n\nIf labels aren't allowed but one is passed as an option, the label will be ignored\nand a warning will be logged.","ref":"cheatsheet.html#labeling"},{"type":"extras","title":"One-Off Executions - Cheatsheet","doc":"In certain cases, there's is only a transient need for sensitive data, such as\nwhen connecting to a system: once the connection is made, the credentials\naren't needed anymore.\n\n```\n{:ok, pid} =\n  SensitiveData.exec(fn ->\n    \"DATABASE_CONNECTION_URI\"\n    |> System.fetch_env!()\n    |> parse_postgres_uri()\n    |> Postgrex.start_link()\n  end)\n```","ref":"cheatsheet.html#one-off-executions"},{"type":"extras","title":"Multiple Wrapper Modules - Cheatsheet","doc":"It can be useful to use several module implementing the `SensitiveData.Wrapper`\nbehaviour, for example to benefit from different redaction capabilities.\n\nLet's take as an example a credit card number which will be converted into a\npayment token which can then be used to charge the user. It would be useful\nto be able to \"peek\" at non-sensitive data (to display to the user in the UI,\nfor debugging, and so on): this can be achieved via redaction. Naturally, as\nthe credit card and associated token are different, they'll require different\nredaction implementations:\n\n```elixir\ndefmodule CreditCard do\n  use SensitiveData.Wrapper, redactor: :redactor\n\n  def redactor(card_number) when is_binary(card_number) do\n    {< >, last_four} = String.split_at(card_number, -4)\n\n    IO.iodata_to_binary([first_number, List.duplicate(\"*\", String.length(to_mask)), last_four])\n  end\nend\n\ndefmodule PaymentToken do\n  use SensitiveData.Wrapper, redactor: :redactor\n\n  def redactor(token) when is_binary(token) do\n    # the first 10 characters aren't considered sensitive\n    {not_sensitive, rest} = String.split_at(token, 10)\n\n    IO.iodata_to_binary([not_sensitive, List.duplicate(\"*\", String.length(rest))])\n  end\nend\n```\n\nFor completeness, let's use the following for tokenization logic:\n\n```elixir\ndefmodule Tokenizer do\n  def tokenize(card_number) when is_binary(card_number) do\n    \"TOK_\" <> random_string(20)\n  end\n\n  defp random_string(length) do\n    :crypto.strong_rand_bytes(length)\n    |> Base.url_encode64\n    |> binary_part(0, length)\n  end\nend\n```\n\nNote how the `Tokenizer` module contains only \"normal\" code: it has no knowledge\nof the existence of any wrapped values.\n\nWe can now easily do the following:\n\n1. obtain sensitive data from the user (their credit card number)\n1. store the sensitive data in a wrapper that still allows us to know\n    enough about the wrapped value\n1. convert the sensitive data to a separate type (and different wrapper)\n    while retaining the ability to inspect non-sensitive portions\n\n```elixir\n    \"Please enter your credit card number: \"\n    SensitiveData.gets_sensitive(into: CreditCard)\n    |> IO.inspect()\n    |> CreditCard.exec(&Tokenizer.tokenize/1, into: PaymentToken)\n```\n\nWhich will output:\n\n```elixir\n#CreditCard \n#PaymentToken \n```\n\nAnd of course we can use guards in our code if we want to ensure only wrapped\ndata of the proper type is used:\n\n```elixir\nimport SensitiveData.Guards, only: [is_sensitive: 2]\n\ndef pay(token) when is_sensitive(token, PaymentToken) do\n  # make the payment\nend\n\ndef pay(credit_card) when is_sensitive(credit_card, CreditCard) do\n  credit_card\n  |> CreditCard.exec(& &1, into: PaymentToken)\n  |> pay()\nend\n```","ref":"cheatsheet.html#multiple-wrapper-modules"},{"type":"extras","title":"Guiding Principles","doc":"# Guiding Principles\n\nEnsuring sensitive data is always handled with the proper care and precautions\ntends to be harder than it appears at first glance.\n\nThe approach taken by this library is that reducing risks is made easier by\nhaving a small set of simple rules to follow: it makes it easier to write safe\ncode, and also facilitates reviewing code for implementations which may be\nunsafe.","ref":"guiding_principles.html"},{"type":"extras","title":"Keep It Simple - Guiding Principles","doc":"At its heart, `SensitiveData` boils down to 2 guiding principles:\n\n- use `c:SensitiveData.Wrapper.from/2` to get sensitive data into a wrapper:\n    sensitive data shouldn't exist outside of `SensitiveData.Wrapper` wrappers\n- use `c:SensitiveData.Wrapper.map/3` and `c:SensitiveData.Wrapper.exec/3` to\n    interact with wrapped sensitive data, so it's protected from inadvertent\n    leaks from stack traces, crash dumps, and similar","ref":"guiding_principles.html#keep-it-simple"},{"type":"extras","title":"Defaults Favor Safety - Guiding Principles","doc":"This library seeks to encourage safer generally-suitable coding practices by\nmaking developers opt-in to features that could leak sensitive data through\ninadvertence.","ref":"guiding_principles.html#defaults-favor-safety"},{"type":"extras","title":"Don't Burden the Supply Chain - Guiding Principles","doc":"This library has no production dependencies and thus won't add to the containing\ncode base's [software supply chain](https://en.wikipedia.org/wiki/Software_supply_chain)\nbeyond itself. This makes auditing easier, as there is no third-party code to review.","ref":"guiding_principles.html#don-t-burden-the-supply-chain"},{"type":"extras","title":"Closing Thoughts - Guiding Principles","doc":"**Using this library in a project does by no means signify that sensitive data\nwon't leak**, nor does it mean developers no longer need to think critically about\ndata handling: there may be [additional mitigations](./data_leak_prevention.html#additional-mitigations)\nyou should consider. That said, this library does aim to make the cognitive\nload associated with handling sensitive data easier to bear by both the people\nwriting code and those reviewing it.","ref":"guiding_principles.html#closing-thoughts"},{"type":"extras","title":"Data Leak Prevention","doc":"# Data Leak Prevention\n\nBy \"data leak\", we mean the unintended exposure of sensitive data outside of\nits intended environment. How can this happen in Elixir and the BEAM? We'll\nlook into the various sensitive data leak, how these can be mitigated, and\nhow `SensitiveData` helps prevent data leaks.\n\nThroughout these examples, we'll use `secret` as a variable whose value we\nconsider sensitive and therefore don't want to leak. This could be\ndatabase credentials, a credit card number, a private cryptography key,\nor anything else you really wouldn't want any unauthorized person to be\nable to see.\n\n```elixir\nsecret = \"SOME SECRET\"\n```\n\nBut before we get into the thick of things, credit where it's due: the\nknowledge distilled here comes in large\npart from the work by the [Erlang Ecosystem Foundation](https://erlef.org/)'s\n[Security Working Group](https://erlef.org/wg/security), most notably their\narticle on\n[protecting sensitive data](https://erlef.github.io/security-wg/secure_coding_and_deployment_hardening/sensitive_data.html).\nPlease refer to the latter for coverage of additional techniques you should be\nusing to protect sensitive data which are beyond the responsibility\nof `SensitiveData`'s scope.","ref":"data_leak_prevention.html"},{"type":"extras","title":"Exceptions and Stack Traces - Data Leak Prevention","doc":"First, let's define a function that will enable us to see what data\nexceptions and stack traces actually contain:\n\n```elixir\ndefmodule Demo do\n    def some_function(arg), do: Map.get(arg, :some_key)\n\n    def inspect_and_reraise(fun) do\n        try do\n            fun.()\n        rescue\n            e ->\n                IO.inspect(e, label: :exception)\n                IO.inspect(__STACKTRACE__, label: :stacktrace)\n                reraise e, __STACKTRACE__\n        end\n    end\nend\n```\n\nThe `some_function/1` function expects a map: what happens if we\ninadvertently call it with a different data type?\n\n```elixir\nDemo.inspect_and_reraise(fn ->\n  Demo.some_function(secret)\nend)\n```\n\nWe're greeted with\n\n```sh\n** (BadMapError) expected a map, got: \"SOME SECRET\"\n    (elixir 1.15.4) lib/map.ex:535: Map.get(\"SOME SECRET\", :some_key, nil)\n    iex:4: (file)\n    iex:4: (file)\n```\n\nOops, our secret is now exposed to stderr, but also in any error tracking tools\nwe may be using. And as you can see in the console, the sensitive data leaks\nfrom both the exceptions, as well as the stack trace:\n\n```sh\nexception: %BadMapError{term: \"SOME SECRET\"}\nstacktrace: [\n  {Map, :get, [\"SOME SECRET\", :some_key, nil],\n   [file: ~c\"lib/map.ex\", line: 535]},\n  {:elixir_eval, :__FILE__, 1, [file: ~c\"iex\", line: 4]},\n  {:elixir, :\"-eval_external_handler/1-fun-2-\", 4,\n   [file: ~c\"src/elixir.erl\", line: 376]},\n  {Util, :inspect_and_reraise, 1, [file: ~c\"iex\", line: 4]},\n  {:elixir, :\"-eval_external_handler/1-fun-2-\", 4,\n   [file: ~c\"src/elixir.erl\", line: 376]},\n  {:erl_eval, :do_apply, 7, [file: ~c\"erl_eval.erl\", line: 750]},\n  {:elixir, :eval_forms, 4, [file: ~c\"src/elixir.erl\", line: 361]},\n  {Module.ParallelChecker, :verify, 1,\n   [file: ~c\"lib/module/parallel_checker.ex\", line: 112]}\n]\n```\n\nSo function calls that deal with sensitive data need to be wrapped in a\n`try`/`rescue` block so that sensitive arguments can be redacted from the\nstack trace, and they must also ensure that sensitive data is removed\nfrom exception structs.\n\n`SensitiveData.exec/2` will do that for you:\n\n```elixir\nDemo.inspect_and_reraise(fn ->\n  SensitiveData.exec(fn ->\n    Demo.some_function(secret)\n  end)\nend)\n```\n\nAnd we'll now only see redacted information:\n\n```sh\nexception: %SensitiveData.RedactedException{exception_name: BadMapError}\nstacktrace: [\n  {Map, :get, 3, [file: ~c\"lib/map.ex\", line: 535]},\n  {:elixir_eval, :__FILE__, 1, [file: ~c\"iex\", line: 2]},\n  {:elixir_eval, :__FILE__, 1, [file: ~c\"iex\", line: 2]},\n  {:elixir, :\"-eval_external_handler/1-fun-2-\", 4,\n   [file: ~c\"src/elixir.erl\", line: 376]},\n  {SensitiveData, :exec, 2, [file: ~c\"lib/sensitive_data.ex\", line: 68]},\n  {:elixir, :\"-eval_external_handler/1-fun-2-\", 4,\n   [file: ~c\"src/elixir.erl\", line: 376]},\n  {Demo, :inspect_and_reraise, 1, [file: ~c\"lib/demo.ex\", line: 6]},\n  {:elixir, :\"-eval_external_handler/1-fun-2-\", 4,\n   [file: ~c\"src/elixir.erl\", line: 376]}\n]\n** (SensitiveData.RedactedException) an exception of type `BadMapError` was raised in a sensitive context\n    (elixir 1.15.4) lib/map.ex:535: Map.get/3\n    iex:2: (file)\n    iex:2: (file)\n    iex:2: (file)\n```","ref":"data_leak_prevention.html#exceptions-and-stack-traces"},{"type":"extras","title":"State and Crashdumps - Data Leak Prevention","doc":"Keep in mind that these examples will use a `GenServer` as an example, but any\nstate-holding process is vulnerable: the same leakage risks exist in `Agent`,\n[`:gen_event`](https://www.erlang.org/doc/man/gen_event),\n[`:gen_statem`](https://www.erlang.org/doc/man/gen_statem), and so on.\n\nTo illustrate how sensitive data can leak from processes, let's use the\nfollowing `GenServer` as an example.\n\n```elixir\ndefmodule Gen do\n  use GenServer\n\n  def init(arg) do\n    {:ok, arg}\n  end\nend\n```\n\n```\nsecret = \"SOME SECRET\"\n\n{:ok, pid} = GenServer.start_link(Gen, secret)\n# {:ok, #PID<0.116.0>}\n\n:observer.start()\n```\n\nHead to the \"Processes\" tab, and sort them descending by Pid via a click on that\ncolumn's header. Locate the pid for our GenServer (`<0.116.0>` in our example),\nright-click on that row and select \"Process info for  \".\n\nWithin the newly opened window for that specific process, head over to the\n\"State\" column: the row with label \"State\" will be proudly displaying our\nsecret plain as day for all to admire.\n\nNaturally, there's an equivalent means to get this directly:\n\n```\n:sys.get_status(pid)\n\n# {:status, #PID<0.116.0>, {:module, :gen_server},\n#  [\n#    [\n#      \"$initial_call\": {Gen, :init, 1},\n#      \"$ancestors\": [#PID<0.110.0>, #PID<0.102.0>]\n#    ],\n#    :running,\n#    #PID<0.110.0>,\n#    [],\n#    [\n#      header: ~c\"Status for generic server <0.116.0>\",\n#      data: [\n#        {~c\"Status\", :running},\n#        {~c\"Parent\", #PID<0.110.0>},\n#        {~c\"Logged events\", []}\n#      ],\n#      data: [{~c\"State\", \"SOME SECRET\"}]\n#    ]\n#  ]}\n```\n\nThat's one way sensitive data could reach unworthy eyes, but we're not quite\ndone ruining our own day yet. \"Let it crash\" is a beautiful philosophy, but\nwhat happens if there's an unrecoverable crash of the BEAM itself? Let's\nfind out.\n\nWe can crash it ourselves (note the argument is a charlist and *not* an Elixir)\nstring:\n\n```\n:erlang.halt('some unrecoverable error')\n```\n\nThis will crash the BEAM and write a crash dump file (typically to\n\"erl_crash.dump\"). Let us know open it up and sift through it:\n\n```\n:crashdump_viewer.start()\n```\n\nSelect the file, and feel free to click \"yes\" when asked if you want to proceed\n(in case of an actual crash you're investigating, make a copy of the crash\ndump first for safekeeping).\n\nIn the Crashdump Viewer, go to the processes tab, and locate the row for the\nGenServer's pid (similarly to the Observer case above), right-click and select\n\"properties for  \".\n\nIn the newly opened window, head over to the \"Stack Dump\" tab and browse the\n\"Term\" column: you'll see one starting with `<<\"SOME_SECRE\"`. Click on it to\nview and you'll once again see the sensitive data exposed.\n\nThis type of leak can be prevented by wrapping the \"SOME_SECRET\" sensitive data\nwithin a structure implementing the `SensitiveData.Wrapper` behaviour. See\n`c:SensitiveData.Wrapper.from/2`.\n\n[//]: # (This is used in an HTML anchor: if updated, update links with)\n[//]: # (#additional-mitigations in the URL)","ref":"data_leak_prevention.html#state-and-crashdumps"},{"type":"extras","title":"Additional Mitigations - Data Leak Prevention","doc":"Your first port of call should be the [Erlang Ecosystem Foundation](https://erlef.org/)'s\narticle on\n[protecting sensitive data](https://erlef.github.io/security-wg/secure_coding_and_deployment_hardening/sensitive_data.html).\n\nWhile wrapping sensitive data within a `SensitiveData.Wrapper` instance will\nconform to many recommendations in the article (wrapping sensitive data in\na closure, pruning stack traces and exception structs, deriving the\n`Inspect` protocol), it doesn't cover others which may be relevant to your\nsituation (such as using the [:private option](https://erlang.org/doc/man/ets.html#new-2)\nfor ETS tables containing sensitive data, flagging the current process as\nsensitive using [:erlang.process_flag(:sensitive, true)](https://erlang.org/doc/man/erlang.html#process_flag-2)\nin processes holding sensitive data or application logic, and so on).","ref":"data_leak_prevention.html#additional-mitigations"}],"content_type":"text/markdown"}